#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""These functions return information about the version of ISIS.

    Most of the time, the only thing you'll probably need is the
    version_info() function which returns an ISISversion tuple.
"""

# Copyright 2019, Ross A. Beyer (rbeyer@seti.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import datetime
import os
import re
from pathlib import Path

from kalasiris import environ


class ISISversion(collections.namedtuple('ISISversion',
                                         ['major', 'minor',
                                          'patch', 'releaselevel',
                                          'date'])):
    '''This is a custom collections.namedtuple() which can contain ISIS version
       information.

       The first three elements, major, minor, and patch should be
       integers.  The fourth element, releaselevel, should be the
       string 'alpha', 'beta', or 'stable' or None.  And the fifth
       element, date, should be a datetime.date object or None.
       That's what the functions in this module will return in an ISISversion
       namedtuple.
    '''


version_re = re.compile(r"(?P<major>\d+)\.(?P<minor>\d+)\.(?P<patch>\d+)")
date_re = re.compile(r"(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})")
date_yearlast_re = re.compile(r"(?P<month>\d{1,2})-(?P<day>\d{1,2})-(?P<year>\d{4})")  # noqa: E501
level_re = re.compile(r"^alpha|beta|stable")


def version_info() -> ISISversion:
    '''Returned named tuple of ISIS version information for the ISIS system
    underlying kalasiris.  If you want to answer, "What version of ISIS is
    being used?"  This is the function you're after.  It is modeled after
    sys.version_info.'''
    return get_from_file(Path(environ['ISISROOT']) / 'version')


def get_from_string(s: str) -> ISISversion:
    '''Read text and parse the contents for ISIS version information.

       This should parse ISIS version text as far back as ISIS 3.5.2.0,
       but possibly earlier.  It will return None values for releaselevel
       and date if it cannot parse them.  It will raise a ValueError if it
       cannot parse a version number.
    '''

    # Version Matching
    match = version_re.search(s)
    if match:
        v = match.groupdict()
    else:
        raise ValueError(f'{s} did not match version regex: '
                         f'{version_re.pattern}')

    # Date Matching
    d = None
    d_match = date_re.search(s)
    if d_match:
        d = d_match.groupdict()
    else:
        d_match = date_yearlast_re.search(s)
        if d_match:
            d = d_match.groupdict()

    if d is not None:
        date = datetime.date(int(d['year']), int(d['month']), int(d['day']))
    else:
        date = None

    # Level Matching
    level = None
    l_match = level_re.search(s)
    if l_match:
        level = l_match.group()

    version = ISISversion(major=int(v['major']),
                          minor=int(v['minor']),
                          patch=int(v['patch']),
                          releaselevel=level,
                          date=date)
    return version


def get_from_file(file_path: os.PathLike) -> ISISversion:
    '''Read an ISIS version file and parse the contents.

       This should parse version files as far back as ISIS 3.5.2.0,
       but possibly earlier.
    '''

    v_text = Path(file_path).read_text()
    return get_from_string(v_text)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `kalasiris` package."""

# Copyright 2019, Ross A. Beyer (rbeyer@seti.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import contextlib
import subprocess
import unittest
from pathlib import Path

import kalasiris as isis
from .utils import resource_check as rc


# Hardcoding these, but I sure would like a better solution.
# IsisPreferences = os.path.join('test-resources', 'IsisPreferences')
HiRISE_img = Path('test-resources') / 'PSP_010502_2090_RED5_0.img'


class TestResources(unittest.TestCase):
    '''Establishes that the test image exists.'''

    def test_resources(self):
        (truth, test) = rc(HiRISE_img)
        self.assertEqual(truth, test)


class Test_getkey_k(unittest.TestCase):

    def setUp(self):
        self.cub = Path('test_getkey_k.cub')
        isis.hi2isis(HiRISE_img, to=self.cub)

    def tearDown(self):
        self.cub.unlink()
        with contextlib.suppress(FileNotFoundError):
            Path('print.prt').unlink()

    def test_getkey_k(self):
        truth = 'HIRISE'
        key = isis.getkey_k(self.cub, 'Instrument', 'InstrumentId')
        self.assertEqual(truth, key)


class Test_hi2isis_k(unittest.TestCase):

    def setUp(self):
        self.img = HiRISE_img

    def tearDown(self):
        with contextlib.suppress(FileNotFoundError):
            Path('print.prt').unlink()

    def test_with_to(self):
        tocube = Path('test_hi2isis_k.cub')
        isis.hi2isis_k(self.img, to=tocube)
        self.assertTrue(tocube.is_file())
        tocube.unlink()

    def test_without_to(self):
        tocube = Path(self.img).with_suffix('.cub')
        isis.hi2isis_k(self.img)
        self.assertTrue(tocube.is_file())
        tocube.unlink()


class Test_hist_k(unittest.TestCase):

    def setUp(self):
        self.cube = Path('test_hist.cub')
        isis.hi2isis(HiRISE_img, to=self.cube)

    def tearDown(self):
        self.cube.unlink()
        with contextlib.suppress(FileNotFoundError):
            Path('print.prt').unlink()

    def test_run(self):
        hist_as_string = isis.hist_k(self.cube)
        self.assertTrue(hist_as_string.startswith('Cube'))

    def test_run_with_to(self):
        text_file = Path('test_hist.hist')
        hist_as_string = isis.hist_k(self.cube, to=text_file)
        self.assertTrue(text_file.is_file())
        self.assertTrue(hist_as_string.startswith('Cube'))
        text_file.unlink()

    def test_fail(self):
        # ISIS hist_k needs at least a FROM=, giving it nothing:
        self.assertRaises(subprocess.CalledProcessError, isis.hist_k)



#!/usr/bin/env python
"""This module contains the PathSet Class.

   Working with ISIS can result in a lot of files to keep track of.
   The PathSet Class is simply a mutable set that only takes Path
   objects.  If you need to keep track of a bunch of files (typically
   to delete them after a set of processing calls), you can use a
   PathSet to keep track of them, and then delete them, like so::

    import kalasiris as isis

    to_delete = isis.PathSet()

    input_p = Path('some.fits')
    output_p = input_p.with_suffix('.done.cub')

    isis_cub = to_delete.add(input_p.with_suffix('.cub'))
    isis.lorri2isis(input_fits, to= isis_cub)

    first = to_delete.add(input_p..with_suffix('.1.cub'))
    isis.some_progeram(isis_cub, to=first)

    second = to_delete.add(input_p..with_suffix('.2.cub'))
    isis.some_program(first, to=second)

    isis.final_step(second, to=output_p)

    to_delete.unlink()
"""

# Copyright 2019, Ross A. Beyer (rbeyer@seti.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path


class PathSet(set):
    """A class for containing a set of Path objects."""

    def __init__(self, iterable=None):
        if iterable:
            for value in iterable:
                if not isinstance(value, Path):
                    raise TypeError('only accepts pathlib.Path objects')
            super().__init__(iterable)
        else:
            super().__init__()

    def add(self, elem):
        '''This variation on add() returns the element.'''
        if not isinstance(elem, Path):
            raise TypeError('only accepts pathlib.Path objects')
        super().add(elem)
        return elem

    def unlink(self):
        '''Just runs Path.unlink() on all members.'''
        for p in self:
            p.unlink()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `PathSet` class."""

# Copyright 2019, Ross A. Beyer (rbeyer@seti.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import contextlib
import unittest
from pathlib import Path

import kalasiris as isis


class TestPathSet(unittest.TestCase):

    def setUp(self):
        self.paths = (Path('test_PathSet_1.cub'),
                      Path('test_PathSet_2.txt'),
                      Path('test_PathSet_3.foo'))

    def tearDown(self):
        with contextlib.suppress(FileNotFoundError):
            for p in self.paths:
                p.unlink()

    def test_init(self):
        ps = isis.PathSet()
        self.assertIsInstance(ps, isis.PathSet)

    def test_init_iterable(self):
        ps = isis.PathSet(self.paths)
        self.assertEqual(3, len(ps))

    def test_init_fail(self):
        self.assertRaises(TypeError, isis.PathSet, 'a string')

    def test_add(self):
        ps = isis.PathSet()
        path1 = self.paths[0]
        added_path = ps.add(path1)
        self.assertEqual(path1, added_path)

    def test_unlink(self):
        ps = isis.PathSet(self.paths)
        for p in ps:
            p.touch()
        ps.unlink()
        for p in ps:
            self.assertFalse(p.exists())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for the `cubenormDialect` class."""

# Copyright 2019, Ross A. Beyer (rbeyer@seti.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import contextlib
import csv
import io
import unittest
from pathlib import Path

import kalasiris as isis
from .utils import resource_check as rc

# Hardcoding this, but I sure would like a better solution.
img = Path('test-resources') / 'PSP_010502_2090_RED5_0.img'


class TestResources(unittest.TestCase):
    '''Establishes that the test image exists.'''

    def test_resources(self):
        (truth, test) = rc(img)
        self.assertEqual(truth, test)


class TestCubenormFile(unittest.TestCase):

    def setUp(self):
        self.cube = Path('test_cubenormfile.cub')
        self.statsfile = Path('test_cubenormfile.stats')
        isis.hi2isis(img, to=self.cube)
        isis.cubenorm(self.cube, stats=self.statsfile)

    def tearDown(self):
        with contextlib.suppress(FileNotFoundError):
            Path('print.prt').unlink()
        self.cube.unlink()
        self.statsfile.unlink()

    def test_Dialect(self):
        with open(self.statsfile) as csvfile:
            reader = csv.reader(csvfile, dialect=isis.cubenormfile.Dialect)
            for row in reader:
                self.assertEqual(8, len(row))
                break

    def test_writer(self):
        columns = list()
        with open(self.statsfile) as csvfile:
            reader = csv.reader(csvfile, dialect=isis.cubenormfile.Dialect)
            for row in reader:
                columns.append(row)

        outfile = self.statsfile.with_suffix('.writer.stats')
        with open(outfile, 'w') as csvfile:
            writer = isis.cubenormfile.writer(csvfile)
            for row in columns:
                writer.writerow(row)

        with open(self.statsfile) as in_file:
            with open(outfile) as out_file:
                for i, o in zip(in_file.readlines(), out_file.readlines()):
                    with self.subTest(inputline=i, outputline=o):
                        self.assertEqual(i, o)
        outfile.unlink()

    def test_DictWriter(self):
        columns = list()
        with open(self.statsfile) as csvfile:
            reader = csv.DictReader(csvfile, dialect=isis.cubenormfile.Dialect)
            for row in reader:
                columns.append(row)

        with io.StringIO() as out_file:
            writer = isis.cubenormfile.DictWriter(out_file)
            writer.writeheader()
            for row in columns:
                writer.writerow(row)

            with open(self.statsfile) as in_file:
                for i, o in zip(in_file.readlines(), out_file.readlines()):
                    with self.subTest(inputline=i, outputline=o):
                        self.assertEqual(i, o)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `Histogram` class."""

# Copyright 2019, Ross A. Beyer (rbeyer@seti.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import contextlib
import unittest
from pathlib import Path

import kalasiris as isis
from .utils import resource_check as rc


# Hardcoding this, but I sure would like a better solution.
HiRISE_img = Path('test-resources') / 'PSP_010502_2090_RED5_0.img'
img = HiRISE_img


class TestResources(unittest.TestCase):
    '''Establishes that the test image exists.'''

    def test_resources(self):
        (truth, test) = rc(img)
        self.assertEqual(truth, test)


class TestHistogram(unittest.TestCase):

    def setUp(self):
        self.cube = Path('test_Histogram.cub')
        self.histfile = Path('test_Histogram.hist')
        isis.hi2isis(img, to=self.cube)
        isis.hist(self.cube, to=self.histfile)

    def tearDown(self):
        with contextlib.suppress(FileNotFoundError):
            Path('print.prt').unlink()
        self.cube.unlink()
        self.histfile.unlink()

    def test_init_histfile(self):
        h = isis.Histogram(self.histfile)
        self.assertIsInstance(h, isis.Histogram)

    def test_init_cube(self):
        h = isis.Histogram(self.cube)
        self.assertIsInstance(h, isis.Histogram)

    def test_init_str(self):
        h = isis.Histogram(isis.hist_k(self.cube))
        self.assertIsInstance(h, isis.Histogram)

    def test_dictlike(self):
        h = isis.Histogram(self.histfile)
        self.assertEqual(self.cube.name, h['Cube'])

    def test_listlike(self):
        h = isis.Histogram(self.histfile)
        self.assertEqual(5, len(h[0]))

    def test_contains(self):
        h = isis.Histogram(self.histfile)
        self.assertTrue('Std Deviation' in h)

    def test_len(self):
        h = isis.Histogram(self.histfile)
        self.assertEqual(107, len(h))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `kalasiris` package."""

# Copyright 2019, Ross A. Beyer (rbeyer@seti.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import importlib
import os
import unittest
import kalasiris.kalasiris


class TestHOME(unittest.TestCase):
    '''Not all platforms have a $HOME environment variable.'''

    def test_HOME(self):
        if 'HOME' in os.environ:
            del os.environ['HOME']

        self.assertTrue(importlib.reload(kalasiris.kalasiris))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2019, Ross A. Beyer (rbeyer@seti.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import csv
import subprocess
from .k_funcs import hist_k


class Histogram(collections.abc.Sequence):
    """Reads the output from ISIS hist and provides it as a sequence.

       The resulting Histogram object primarily behaves like a list, where
       that list represents the rows of the ISIS hist output.  Each of those
       rows is a ``namedtuple`` which contains the elements of each row,
       referenced by their column names.

       The Histogram object also has some dictionary-like capabilities, in
       order to get at the values listed in the ISIS hist output in the
       section before the numerical output.
    """

    def __init__(self, histinfo):
        self.histinfo = histinfo

        try:
            (self.dictionary, self.headers, self.hist_list) \
                = self.parse(histinfo)
        except StopIteration:
            try:
                (self.dictionary, self.headers, self.hist_list) \
                    = self.parse(hist_k(histinfo))
            except subprocess.CalledProcessError:
                with open(histinfo, 'r') as f:
                    (self.dictionary, self.headers, self.hist_list) \
                        = self.parse(f.read())

    def __str__(self):
        return(str(self.dictionary))

    def __repr__(self):
        return (f'{self.__class__.__name__}(\'{self.histfile}\')')

    def __len__(self):
        return len(self.hist_list)

    def __getitem__(self, key):
        try:
            return self.dictionary[key]
        except KeyError:
            return self.hist_list[key]

    def __iter__(self):
        return self.hist_list.__iter__()

    def __contains__(self, item):
        if item in self.dictionary:
            return True
        else:
            return(item in self.hist_list)

    def keys(self):
        '''Gets the keys from the initial portion of the hist output file.

           These will be items like 'Cube', 'Band', 'Average', etc.
        '''
        return self.dictionary.keys()

    def values(self):
        '''Gets the values from the initial portion of the hist output file.'''
        return self.dictionary.values()

    @staticmethod
    def parse(histinfo: str) -> tuple:
        '''Takes a string (expecting the output of ISIS ``hist``), and
           parses the output.

           A three-element namedtuple is returned: the first element
           is a *dictionary* of the name:value information at the
           top of the file, the second element is a *list* of the
           of the fields that decorate the top of the histogram
           rows, and the third element is a *list* of HistRow
           namedtuples that represent each row of the hist output.

           The contents of the file that results from ISIS ``hist``
           look like this::

                Cube:           PSP_010502_2090_RED5_0.EDR_Stats.cub
                Band:           1
                Average:        6490.68
                [... other lines like this ...]
                His Pixels:      0
                Hrs Pixels:      0


                DN,Pixels,CumulativePixels,Percent,CumulativePercent
                3889,1,1,4.88281e-05,4.88281e-05
                3924,1,2,4.88281e-05,9.76563e-05
                3960,2,4,9.76563e-05,0.000195313
                [... more comma-separated lines like above ...]

           But this function sees it like this::

                k: v
                k: v
                k: v
                [... other lines like this ...]
                k: v
                k: v


                h,h,h,h,h
                n,n,n,n,n
                n,n,n,n,n
                n,n,n,n,n
                [... more comma-separated lines like above ...]

           Where each of the letters above is a string value that the parser
           reads.

           First, it takes all of the ``k`` and ``v`` elements and saves them
           as the keys and values in the returned dictionary.

           Second, the ``h`` elements are returned them as the list of
           fieldnames.

           Third, it reads the lines with ``n`` and stores them as
           ``namedtuples`` in the returned list.
        '''
        d = dict()
        fieldnames = []
        hist_rows = []
        for line in filter(lambda x: ':' in x, str(histinfo).splitlines()):
            (k, v) = line.split(':')
            # d[k.strip()] = v.strip()
            d.setdefault(k.strip(), v.strip())

        reader = csv.reader(filter(lambda x: ',' in x,
                                   str(histinfo).splitlines()))
        fieldnames = next(reader)

        HistRow = collections.namedtuple('HistRow', fieldnames)
        for row in map(HistRow._make, reader):
            hist_rows.append(row)

        HistParsed = collections.namedtuple('HistParsed',
                                            ['info', 'fieldnames', 'data'])
        return(HistParsed(d, fieldnames, hist_rows))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `sweetened` module."""

# Copyright 2019, Ross A. Beyer (rbeyer@seti.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import contextlib
import shutil
import unittest
from pathlib import Path

import kalasiris.sweetened as isis
from .utils import resource_check as rc


# Hardcoding this, but I sure would like a better solution.
HiRISE_img = Path('test-resources') / 'PSP_010502_2090_RED5_0.img'
img = HiRISE_img


class TestResources(unittest.TestCase):
    '''Establishes that the test image exists.'''

    def test_resources(self):
        (truth, test) = rc(img)
        self.assertEqual(truth, test)


class Test_hi2isis(unittest.TestCase):

    def setUp(self):
        self.img = img

    def tearDown(self):
        with contextlib.suppress(FileNotFoundError):
            Path('print.prt').unlink()

    def test_hi2isis_with_to(self):
        tocube = Path('test_sweetened_hi2isis.cub')
        isis.hi2isis(self.img, to=tocube)
        self.assertTrue(tocube.is_file())
        tocube.unlink()

    def test_hi2isis_without_to(self):
        sweet_img = self.img.with_name('sweet.img')
        shutil.copy(self.img, sweet_img)
        tocube = sweet_img.with_suffix('.cub')
        isis.hi2isis(sweet_img)
        self.assertTrue(tocube.is_file())
        tocube.unlink()
        sweet_img.unlink()


class Test_getkey(unittest.TestCase):

    def setUp(self):
        self.cub = Path('test_sweetened_getkey.cub')
        isis.hi2isis(HiRISE_img, to=self.cub)

    def tearDown(self):
        self.cub.unlink()
        with contextlib.suppress(FileNotFoundError):
            Path('print.prt').unlink()

    def test_getkey_k(self):
        truth = 'HIRISE'
        key = isis.getkey(self.cub, 'Instrument', 'InstrumentId')
        self.assertEqual(truth, key)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Provides kalasiris *_k functions*.

   The kalasiris *_k functions* provide some syntactic sugar to make
   calling the ISIS programs just that much easier.  For example here
   are two ways to do the same thing::

        import kalasiris as isis

        cube_file = 'some.cub'

        keyval = isis.getkey(cube_file, grpname='Instrument',
                         keyword='InstrumentId').stdout.strip()

        k_keyval = isis.getkey_k(cube_file, 'Instrument', 'InstrumentId')

   And the values of ``keyval`` and ``k_keyval`` are identical, its
   just that the *_k function* version is a little more compact.
   Each of the *_k functions* implements their modifications a
   little differently, so make sure to read their documentation.
"""

# Copyright 2019, Ross A. Beyer (rbeyer@seti.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import tempfile
import os
from pathlib import Path

import kalasiris as isis


def getkey_k(cube: os.PathLike, group: str, key: str) -> str:
    '''Simplified calling for getkey.

    No default parameters are needed, and it directly returns a string.
    '''
    return(isis.getkey(cube, grpname=group, keyword=key).stdout.strip())


def hi2isis_k(*args, **kwargs):
    '''Creates a default name for the to= cube.

       If the FROM file has the name ``foo.img``, then the output will
       be ``foo.cub``.
    '''
    from_path = Path()
    if len(args) > 0 and not str(args[0]).endswith('__'):
        from_path = Path(args[0])
    else:
        for(k, v) in kwargs.items():
            if 'from_' == k:
                from_path = Path(v)
    if 'to_' not in kwargs:
        kwargs.setdefault('to', from_path.with_suffix('.cub'))
    return(isis.hi2isis(*args, **kwargs))


def hist_k(*args, **kwargs) -> str:
    '''Returns the contents of the file created by ISIS hist as a string.

       If there is a TO= parameter in the arguments, ``hist_k()`` will
       create the file, and return its contents as a string
    '''
    to_pathlike = None
    for (k, v) in kwargs.items():
        if 'to' == k or 'to_' == k:
            to_pathlike = v

    f = None
    if not to_pathlike:
        f = tempfile.NamedTemporaryFile(mode='w+')
        kwargs['to'] = f.name

    isis.hist(*args, **kwargs)

    if not f:
        f = open(to_pathlike, 'r')
    contents = f.read()
    f.close()

    return contents



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""The ISIS cubenorm file outputs plain text table information and
   also reads it in, but the format is a very specific fixed-width
   table format.  A plain csv.reader or csv.DictReader using the
   cubenormfile.Dialect object will be able to read the text output
   of cubenorm, but to write out a file that cubenorm will read in,
   you will need to use the cubenormfile.writer or
   cubenormfile.DictWriter classes.
"""

# Copyright 2019, Ross A. Beyer (rbeyer@seti.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import csv

# Establish the allowable cubenorm fieldnames and their character widths.
fieldnames = ('Band', 'RowCol', 'ValidPoints', 'Average',
              'Median', 'StdDev', 'Minimum', 'Maximum')

# These widths are extremely fragile, and if cubenorm changes,
# then these will need to be changed, too.
fieldwidth = dict()
for n in fieldnames:
    fieldwidth.setdefault(n, 15)
fieldwidth['Band'] = 8
fieldwidth['RowCol'] = 8


class Dialect(csv.Dialect):
    '''A csv.Dialect for the output of the ISIS cubenorm program.'''
    delimiter = ' '
    skipinitialspace = True
    quoting = csv.QUOTE_NONE
    escapechar = ''
    lineterminator = '\n'


class writer:
    '''A class for writing out the fixed-width format required by cubenorm.

    The interface is similar to the csv.writer class, but does not inheirit
    from it.'''

    def __init__(self, f):
        self.file_object = f

    def writerow(self, row):
        line = ''
        for name, elem in zip(fieldnames, row):
            right_aligned = '{:>' + str(fieldwidth[name]) + '}'
            line += right_aligned.format(elem)

        self.file_object.write(line + '\n')

    def writerows(self, rows):
        for r in rows:
            self.writerow(r)

    def writeheader(self):
        '''A convenence function, since the fieldnames are pre-defined.'''
        self.writerow(fieldnames)


class DictWriter(csv.DictWriter):
    '''A DictWriter for cubenorm files.'''

    def __init__(self, f, restval="", extrasaction="raise",
                 dialect=Dialect, *args, **kwds):
        self.fieldnames = fieldnames
        self.restval = restval
        if extrasaction.lower() not in ("raise", "ignore"):
            raise ValueError("extrasaction (%s) must be 'raise' or 'ignore'"
                             % extrasaction)
        self.extrasaction = extrasaction
        self.writer = writer(f)
